@@ -79,3 +79,15 @@ _ = f"a {f"first"
     + f"second"} d"
 _ = f"a {f"first {f"middle"}"
     + f"second"} d"
+
+# See https://github.com/astral-sh/ruff/issues/12936
+_ = "\12""0" # fix should be "\0120"
+_ = "\\12""0" # fix should be "\\120"
+_ = "\\\12""0" # fix should be "\\\0120"
+_ = "\12 0""0" # fix should be "\12 00"
+_ = r"\12"r"0" # fix should be r"\120"
+_ = "\12 and more""0" # fix should be "\12 and more0"
+_ = "\8""0" # fix should be "\80"
+_ = "\12""8" # fix should be "\128"
+_ = "\12""foo" # fix should be "\12foo"
+_ = "\12" ""  # fix should be "\12"
