@@ -1404,6 +1404,9 @@ pub(crate) fn expression(expr: &Expr, checker: &mut Checker) {
             if checker.enabled(Rule::IfExpInsteadOfOrOperator) {
                 refurb::rules::if_exp_instead_of_or_operator(checker, if_exp);
             }
+            if checker.enabled(Rule::UselessIfElse) {
+                ruff::rules::useless_if_else(checker, if_exp);
+            }
         }
         Expr::ListComp(
             comp @ ast::ExprListComp {
