@@ -13,10 +13,9 @@ impl Emitter for JsonLinesEmitter {
         messages: &[Message],
         context: &EmitterContext,
     ) -> anyhow::Result<()> {
-        let mut w = writer;
         for message in messages {
-            serde_json::to_writer(&mut w, &message_to_json_value(message, context))?;
-            w.write_all(b"\n")?;
+            serde_json::to_writer(&mut *writer, &message_to_json_value(message, context))?;
+            writer.write_all(b"\n")?;
         }
         Ok(())
     }
@@ -29,7 +28,7 @@ mod tests {
     use crate::message::json_lines::JsonLinesEmitter;
     use crate::message::tests::{
         capture_emitter_notebook_output, capture_emitter_output, create_messages,
-        create_notebook_messages,
+        create_notebook_messages, create_syntax_error_messages,
     };
 
     #[test]
@@ -40,6 +39,14 @@ mod tests {
         assert_snapshot!(content);
     }
 
+    #[test]
+    fn syntax_errors() {
+        let mut emitter = JsonLinesEmitter;
+        let content = capture_emitter_output(&mut emitter, &create_syntax_error_messages());
+
+        assert_snapshot!(content);
+    }
+
     #[test]
     fn notebook_output() {
         let mut emitter = JsonLinesEmitter;
