@@ -4,28 +4,27 @@ use ruff_python_ast::Stmt;
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
-use crate::registry::Rule;
 
 /// ## What it does
 /// Checks for `pass` statements in empty stub bodies.
 ///
 /// ## Why is this bad?
-/// For consistency, empty stub bodies should contain `...` instead of `pass`.
-///
-/// Additionally, an ellipsis better conveys the intent of the stub body (that
-/// the body has been implemented, but has been intentionally left blank to
-/// document the interface).
+/// For stylistic consistency, `...` should always be used rather than `pass`
+/// in stub files.
 ///
 /// ## Example
-/// ```python
-/// def foo(bar: int) -> list[int]:
-///     pass
+/// ```pyi
+/// def foo(bar: int) -> list[int]: pass
 /// ```
 ///
 /// Use instead:
-/// ```python
+/// ```pyi
 /// def foo(bar: int) -> list[int]: ...
 /// ```
+///
+/// ## References
+/// The [recommended style for functions and methods](https://typing.readthedocs.io/en/latest/source/stubs.html#functions-and-methods)
+/// in the typing docs.
 #[violation]
 pub struct PassStatementStubBody;
 
@@ -47,11 +46,9 @@ pub(crate) fn pass_statement_stub_body(checker: &mut Checker, body: &[Stmt]) {
     };
 
     let mut diagnostic = Diagnostic::new(PassStatementStubBody, pass.range());
-    if checker.patch(Rule::PassStatementStubBody) {
-        diagnostic.set_fix(Fix::automatic(Edit::range_replacement(
-            format!("..."),
-            pass.range(),
-        )));
-    };
+    diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
+        format!("..."),
+        pass.range(),
+    )));
     checker.diagnostics.push(diagnostic);
 }
