@@ -4,8 +4,10 @@ use ruff_python_ast::identifier::Identifier;
 use ruff_python_ast::visitor::source_order;
 use ruff_python_ast::{self as ast, AnyNodeRef, Expr, Stmt};
 use ruff_python_semantic::analyze::function_type::is_stub;
+use ruff_python_semantic::Modules;
 
 use crate::checkers::ast::Checker;
+use crate::rules::fastapi::rules::is_fastapi_route;
 
 /// ## What it does
 /// Checks for functions declared `async` that do not await or otherwise use features requiring the
@@ -173,6 +175,12 @@ pub(crate) fn unused_async(
         return;
     }
 
+    if checker.semantic().seen_module(Modules::FASTAPI)
+        && is_fastapi_route(function_def, checker.semantic())
+    {
+        return;
+    }
+
     let found_await_or_async = {
         let mut visitor = AsyncExprVisitor::default();
         source_order::walk_body(&mut visitor, body);
