@@ -111,13 +111,8 @@ pub(crate) fn raise_within_try(checker: &mut Checker, body: &[Stmt], handlers: &
             || handled_exceptions.iter().any(|expr| {
                 checker
                     .semantic()
-                    .resolve_qualified_name(expr)
-                    .is_some_and(|qualified_name| {
-                        matches!(
-                            qualified_name.segments(),
-                            ["", "Exception" | "BaseException"]
-                        )
-                    })
+                    .resolve_builtin_symbol(expr)
+                    .is_some_and(|builtin| matches!(builtin, "Exception" | "BaseException"))
             })
         {
             checker
