@@ -10,28 +10,26 @@ use itertools::Itertools;
 use rustc_hash::FxHashMap;
 
 use ruff_diagnostics::{Applicability, Diagnostic, FixAvailability};
+use ruff_notebook::Notebook;
+#[cfg(not(fuzzing))]
+use ruff_notebook::NotebookError;
 use ruff_python_ast::PySourceType;
 use ruff_python_codegen::Stylist;
 use ruff_python_index::Indexer;
-use ruff_python_parser::lexer::LexResult;
-use ruff_python_parser::AsMode;
+use ruff_python_parser::ParseError;
 use ruff_python_trivia::textwrap::dedent;
 use ruff_source_file::{Locator, SourceFileBuilder};
 use ruff_text_size::Ranged;
 
 use crate::directives;
 use crate::fix::{fix_file, FixResult};
-use crate::linter::{check_path, LinterResult, TokenSource};
+use crate::linter::check_path;
 use crate::message::{Emitter, EmitterContext, Message, TextEmitter};
 use crate::packaging::detect_package_root;
 use crate::registry::AsRule;
-use crate::rules::pycodestyle::rules::syntax_error;
 use crate::settings::types::UnsafeFixes;
 use crate::settings::{flags, LinterSettings};
 use crate::source_kind::SourceKind;
-use ruff_notebook::Notebook;
-#[cfg(not(fuzzing))]
-use ruff_notebook::NotebookError;
 
 #[cfg(not(fuzzing))]
 pub(crate) fn test_resource_path(path: impl AsRef<Path>) -> std::path::PathBuf {
@@ -92,7 +90,7 @@ pub fn test_snippet(contents: &str, settings: &LinterSettings) -> Vec<Message> {
 }
 
 thread_local! {
-    static MAX_ITERATIONS: std::cell::Cell<usize> = const { std::cell::Cell::new(8) };
+    static MAX_ITERATIONS: std::cell::Cell<usize> = const { std::cell::Cell::new(10) };
 }
 
 pub fn set_max_iterations(max: usize) {
@@ -111,21 +109,17 @@ pub(crate) fn test_contents<'a>(
     settings: &LinterSettings,
 ) -> (Vec<Message>, Cow<'a, SourceKind>) {
     let source_type = PySourceType::from(path);
-    let tokens: Vec<LexResult> =
-        ruff_python_parser::tokenize(source_kind.source_code(), source_type.as_mode());
+    let parsed = ruff_python_parser::parse_unchecked_source(source_kind.source_code(), source_type);
     let locator = Locator::new(source_kind.source_code());
-    let stylist = Stylist::from_tokens(&tokens, &locator);
-    let indexer = Indexer::from_tokens(&tokens, &locator);
+    let stylist = Stylist::from_tokens(parsed.tokens(), &locator);
+    let indexer = Indexer::from_tokens(parsed.tokens(), &locator);
     let directives = directives::extract_directives(
-        &tokens,
+        parsed.tokens(),
         directives::Flags::from_settings(settings),
         &locator,
         &indexer,
     );
-    let LinterResult {
-        data: (diagnostics, _imports),
-        error,
-    } = check_path(
+    let diagnostics = check_path(
         path,
         path.parent()
             .and_then(|parent| detect_package_root(parent, &settings.namespace_packages)),
@@ -137,10 +131,10 @@ pub(crate) fn test_contents<'a>(
         flags::Noqa::Enabled,
         source_kind,
         source_type,
-        TokenSource::Tokens(tokens),
+        &parsed,
     );
 
-    let source_has_errors = error.is_some();
+    let source_has_errors = !parsed.is_valid();
 
     // Detect fixes that don't converge after multiple iterations.
     let mut iterations = 0;
@@ -177,22 +171,19 @@ pub(crate) fn test_contents<'a>(
 
             transformed = Cow::Owned(transformed.updated(fixed_contents, &source_map));
 
-            let tokens: Vec<LexResult> =
-                ruff_python_parser::tokenize(transformed.source_code(), source_type.as_mode());
+            let parsed =
+                ruff_python_parser::parse_unchecked_source(transformed.source_code(), source_type);
             let locator = Locator::new(transformed.source_code());
-            let stylist = Stylist::from_tokens(&tokens, &locator);
-            let indexer = Indexer::from_tokens(&tokens, &locator);
+            let stylist = Stylist::from_tokens(parsed.tokens(), &locator);
+            let indexer = Indexer::from_tokens(parsed.tokens(), &locator);
             let directives = directives::extract_directives(
-                &tokens,
+                parsed.tokens(),
                 directives::Flags::from_settings(settings),
                 &locator,
                 &indexer,
             );
 
-            let LinterResult {
-                data: (fixed_diagnostics, _),
-                error: fixed_error,
-            } = check_path(
+            let fixed_diagnostics = check_path(
                 path,
                 None,
                 &locator,
@@ -203,28 +194,24 @@ pub(crate) fn test_contents<'a>(
                 flags::Noqa::Enabled,
                 &transformed,
                 source_type,
-                TokenSource::Tokens(tokens),
+                &parsed,
             );
 
-            if let Some(fixed_error) = fixed_error {
-                if !source_has_errors {
-                    // Previous fix introduced a syntax error, abort
-                    let fixes = print_diagnostics(diagnostics, path, source_kind);
+            if !parsed.is_valid() && !source_has_errors {
+                // Previous fix introduced a syntax error, abort
+                let fixes = print_diagnostics(diagnostics, path, source_kind);
+                let syntax_errors =
+                    print_syntax_errors(parsed.errors(), path, &locator, &transformed);
 
-                    let mut syntax_diagnostics = Vec::new();
-                    syntax_error(&mut syntax_diagnostics, &fixed_error, &locator);
-                    let syntax_errors = print_diagnostics(syntax_diagnostics, path, &transformed);
-
-                    panic!(
-                        r#"Fixed source has a syntax error where the source document does not. This is a bug in one of the generated fixes:
+                panic!(
+                    "Fixed source has a syntax error where the source document does not. This is a bug in one of the generated fixes:
 {syntax_errors}
 Last generated fixes:
 {fixes}
 Source with applied fixes:
-{}"#,
-                        transformed.source_code()
-                    );
-                }
+{}",
+                    transformed.source_code()
+                );
             }
 
             diagnostics = fixed_diagnostics;
@@ -241,7 +228,12 @@ Source with applied fixes:
         .into_iter()
         .map(|diagnostic| {
             let rule = diagnostic.kind.rule();
-            let fixable = diagnostic.fix.as_ref().is_some_and(|fix| matches!(fix.applicability(), Applicability::Safe | Applicability::Unsafe));
+            let fixable = diagnostic.fix.as_ref().is_some_and(|fix| {
+                matches!(
+                    fix.applicability(),
+                    Applicability::Safe | Applicability::Unsafe
+                )
+            });
 
             match (fixable, rule.fixable()) {
                 (true, FixAvailability::Sometimes | FixAvailability::Always)
@@ -249,25 +241,65 @@ Source with applied fixes:
                     // Ok
                 }
                 (true, FixAvailability::None) => {
-                    panic!("Rule {rule:?} is marked as non-fixable but it created a fix. Change the `Violation::FIX_AVAILABILITY` to either `FixAvailability::Sometimes` or `FixAvailability::Always`");
-                },
+                    panic!(
+                        "Rule {rule:?} is marked as non-fixable but it created a fix.
+Change the `Violation::FIX_AVAILABILITY` to either \
+`FixAvailability::Sometimes` or `FixAvailability::Always`"
+                    );
+                }
+                (false, FixAvailability::Always) if source_has_errors => {
+                    // Ok
+                }
                 (false, FixAvailability::Always) => {
-                    panic!("Rule {rule:?} is marked to always-fixable but the diagnostic has no fix. Either ensure you always emit a fix or change `Violation::FIX_AVAILABILITY` to either `FixAvailability::Sometimes` or `FixAvailability::None")
+                    panic!(
+                        "\
+Rule {rule:?} is marked to always-fixable but the diagnostic has no fix.
+Either ensure you always emit a fix or change `Violation::FIX_AVAILABILITY` to either \
+`FixAvailability::Sometimes` or `FixAvailability::None`"
+                    )
                 }
             }
 
-            assert!(!(fixable && diagnostic.kind.suggestion.is_none()), "Diagnostic emitted by {rule:?} is fixable but `Violation::fix_title` returns `None`.`");
+            assert!(
+                !(fixable && diagnostic.kind.suggestion.is_none()),
+                "Diagnostic emitted by {rule:?} is fixable but \
+                `Violation::fix_title` returns `None`"
+            );
 
             // Not strictly necessary but adds some coverage for this code path
             let noqa = directives.noqa_line_for.resolve(diagnostic.start());
 
             Message::from_diagnostic(diagnostic, source_code.clone(), noqa)
         })
+        .chain(parsed.errors().iter().map(|parse_error| {
+            Message::from_parse_error(parse_error, &locator, source_code.clone())
+        }))
         .sorted()
         .collect();
     (messages, transformed)
 }
 
+fn print_syntax_errors(
+    errors: &[ParseError],
+    path: &Path,
+    locator: &Locator,
+    source: &SourceKind,
+) -> String {
+    let filename = path.file_name().unwrap().to_string_lossy();
+    let source_file = SourceFileBuilder::new(filename.as_ref(), source.source_code()).finish();
+
+    let messages: Vec<_> = errors
+        .iter()
+        .map(|parse_error| Message::from_parse_error(parse_error, locator, source_file.clone()))
+        .collect();
+
+    if let Some(notebook) = source.as_ipy_notebook() {
+        print_jupyter_messages(&messages, path, notebook)
+    } else {
+        print_messages(&messages)
+    }
+}
+
 fn print_diagnostics(diagnostics: Vec<Diagnostic>, path: &Path, source: &SourceKind) -> String {
     let filename = path.file_name().unwrap().to_string_lossy();
     let source_file = SourceFileBuilder::new(filename.as_ref(), source.source_code()).finish();
