@@ -4,6 +4,7 @@ mod branches;
 mod context;
 mod definition;
 mod globals;
+mod imports;
 mod model;
 mod nodes;
 mod reference;
@@ -15,6 +16,7 @@ pub use branches::*;
 pub use context::*;
 pub use definition::*;
 pub use globals::*;
+pub use imports::*;
 pub use model::*;
 pub use nodes::*;
 pub use reference::*;
