@@ -2,10 +2,10 @@ use lsp_types::{self as types, request as req};
 use types::TextEdit;
 
 use ruff_source_file::LineIndex;
-use ruff_workspace::resolver::match_any_exclusion;
 
 use crate::edit::{Replacement, ToRangeExt};
 use crate::fix::Fixes;
+use crate::resolve::is_document_excluded;
 use crate::server::api::LSPResult;
 use crate::server::{client::Notifier, Result};
 use crate::session::{DocumentQuery, DocumentSnapshot};
@@ -85,14 +85,12 @@ fn format_text_document(
 
     // If the document is excluded, return early.
     if let Some(file_path) = query.file_path() {
-        if let Some(exclusion) = match_any_exclusion(
+        if is_document_excluded(
             &file_path,
-            &file_resolver_settings.exclude,
-            &file_resolver_settings.extend_exclude,
+            file_resolver_settings,
             None,
-            Some(&formatter_settings.exclude),
+            Some(formatter_settings),
         ) {
-            tracing::debug!("Ignored path via `{}`: {}", exclusion, file_path.display());
             return Ok(None);
         }
     }
