@@ -21,6 +21,7 @@ pub(crate) struct ResolvedClientSettings {
     lint_enable: bool,
     disable_rule_comment_enable: bool,
     fix_violation_enable: bool,
+    show_syntax_errors: bool,
     editor_settings: ResolvedEditorSettings,
 }
 
@@ -60,7 +61,7 @@ pub(crate) enum ConfigurationPreference {
 #[derive(Debug, Deserialize, Default)]
 #[cfg_attr(test, derive(PartialEq, Eq))]
 #[serde(rename_all = "camelCase")]
-pub(crate) struct ClientSettings {
+pub struct ClientSettings {
     configuration: Option<String>,
     fix_all: Option<bool>,
     organize_imports: Option<bool>,
@@ -70,6 +71,42 @@ pub(crate) struct ClientSettings {
     exclude: Option<Vec<String>>,
     line_length: Option<LineLength>,
     configuration_preference: Option<ConfigurationPreference>,
+
+    /// If `true` or [`None`], show syntax errors as diagnostics.
+    ///
+    /// This is useful when using Ruff with other language servers, allowing the user to refer
+    /// to syntax errors from only one source.
+    show_syntax_errors: Option<bool>,
+
+    // These settings are only needed for tracing, and are only read from the global configuration.
+    // These will not be in the resolved settings.
+    #[serde(flatten)]
+    pub(crate) tracing: TracingSettings,
+}
+
+impl ClientSettings {
+    /// Update the preview flag for the linter and the formatter with the given value.
+    pub(crate) fn set_preview(&mut self, preview: bool) {
+        match self.lint.as_mut() {
+            None => self.lint = Some(LintOptions::default().with_preview(preview)),
+            Some(lint) => lint.set_preview(preview),
+        }
+        match self.format.as_mut() {
+            None => self.format = Some(FormatOptions::default().with_preview(preview)),
+            Some(format) => format.set_preview(preview),
+        }
+    }
+}
+
+/// Settings needed to initialize tracing. These will only be
+/// read from the global configuration.
+#[derive(Debug, Deserialize, Default)]
+#[cfg_attr(test, derive(PartialEq, Eq))]
+#[serde(rename_all = "camelCase")]
+pub(crate) struct TracingSettings {
+    pub(crate) log_level: Option<crate::trace::LogLevel>,
+    /// Path to the log file - tildes and environment variables are supported.
+    pub(crate) log_file: Option<PathBuf>,
 }
 
 /// This is a direct representation of the workspace settings schema,
@@ -84,7 +121,7 @@ struct WorkspaceSettings {
     workspace: Url,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Debug, Default, Deserialize)]
 #[cfg_attr(test, derive(PartialEq, Eq))]
 #[serde(rename_all = "camelCase")]
 struct LintOptions {
@@ -95,6 +132,17 @@ struct LintOptions {
     ignore: Option<Vec<String>>,
 }
 
+impl LintOptions {
+    fn with_preview(mut self, preview: bool) -> LintOptions {
+        self.preview = Some(preview);
+        self
+    }
+
+    fn set_preview(&mut self, preview: bool) {
+        self.preview = Some(preview);
+    }
+}
+
 #[derive(Debug, Default, Deserialize)]
 #[cfg_attr(test, derive(PartialEq, Eq))]
 #[serde(rename_all = "camelCase")]
@@ -102,6 +150,17 @@ struct FormatOptions {
     preview: Option<bool>,
 }
 
+impl FormatOptions {
+    fn with_preview(mut self, preview: bool) -> FormatOptions {
+        self.preview = Some(preview);
+        self
+    }
+
+    fn set_preview(&mut self, preview: bool) {
+        self.preview = Some(preview);
+    }
+}
+
 #[derive(Debug, Default, Deserialize)]
 #[cfg_attr(test, derive(PartialEq, Eq))]
 #[serde(rename_all = "camelCase")]
@@ -136,6 +195,7 @@ enum InitializationOptions {
 }
 
 /// Built from the initialization options provided by the client.
+#[derive(Debug)]
 pub(crate) struct AllSettings {
     pub(crate) global_settings: ClientSettings,
     /// If this is `None`, the client only passed in global settings.
@@ -156,6 +216,16 @@ impl AllSettings {
         )
     }
 
+    /// Update the preview flag for both the global and all workspace settings.
+    pub(crate) fn set_preview(&mut self, preview: bool) {
+        self.global_settings.set_preview(preview);
+        if let Some(workspace_settings) = self.workspace_settings.as_mut() {
+            for settings in workspace_settings.values_mut() {
+                settings.set_preview(preview);
+            }
+        }
+    }
+
     fn from_init_options(options: InitializationOptions) -> Self {
         let (global_settings, workspace_settings) = match options {
             InitializationOptions::GlobalOnly { settings } => (settings, None),
@@ -229,6 +299,11 @@ impl ResolvedClientSettings {
                 },
                 true,
             ),
+            show_syntax_errors: Self::resolve_or(
+                all_settings,
+                |settings| settings.show_syntax_errors,
+                true,
+            ),
             editor_settings: ResolvedEditorSettings {
                 configuration: Self::resolve_optional(all_settings, |settings| {
                     settings
@@ -330,6 +405,10 @@ impl ResolvedClientSettings {
         self.fix_violation_enable
     }
 
+    pub(crate) fn show_syntax_errors(&self) -> bool {
+        self.show_syntax_errors
+    }
+
     pub(crate) fn editor_settings(&self) -> &ResolvedEditorSettings {
         &self.editor_settings
     }
@@ -361,6 +440,11 @@ mod tests {
     const EMPTY_INIT_OPTIONS_FIXTURE: &str =
         include_str!("../../resources/test/fixtures/settings/empty.json");
 
+    // This fixture contains multiple workspaces with empty initialization options. It only sets
+    // the `cwd` and the `workspace` value.
+    const EMPTY_MULTIPLE_WORKSPACE_INIT_OPTIONS_FIXTURE: &str =
+        include_str!("../../resources/test/fixtures/settings/empty_multiple_workspace.json");
+
     fn deserialize_fixture<T: DeserializeOwned>(content: &str) -> T {
         serde_json::from_str(content).expect("test fixture JSON should deserialize")
     }
@@ -424,6 +508,13 @@ mod tests {
                 exclude: None,
                 line_length: None,
                 configuration_preference: None,
+                show_syntax_errors: Some(
+                    true,
+                ),
+                tracing: TracingSettings {
+                    log_level: None,
+                    log_file: None,
+                },
             },
             workspace_settings: [
                 WorkspaceSettings {
@@ -472,6 +563,13 @@ mod tests {
                         exclude: None,
                         line_length: None,
                         configuration_preference: None,
+                        show_syntax_errors: Some(
+                            true,
+                        ),
+                        tracing: TracingSettings {
+                            log_level: None,
+                            log_file: None,
+                        },
                     },
                     workspace: Url {
                         scheme: "file",
@@ -533,6 +631,13 @@ mod tests {
                         exclude: None,
                         line_length: None,
                         configuration_preference: None,
+                        show_syntax_errors: Some(
+                            true,
+                        ),
+                        tracing: TracingSettings {
+                            log_level: None,
+                            log_file: None,
+                        },
                     },
                     workspace: Url {
                         scheme: "file",
@@ -575,6 +680,7 @@ mod tests {
                 lint_enable: true,
                 disable_rule_comment_enable: false,
                 fix_violation_enable: false,
+                show_syntax_errors: true,
                 editor_settings: ResolvedEditorSettings {
                     configuration: None,
                     lint_preview: Some(true),
@@ -588,7 +694,7 @@ mod tests {
                     exclude: None,
                     line_length: None,
                     configuration_preference: ConfigurationPreference::default(),
-                }
+                },
             }
         );
         let path =
@@ -606,6 +712,7 @@ mod tests {
                 lint_enable: true,
                 disable_rule_comment_enable: true,
                 fix_violation_enable: false,
+                show_syntax_errors: true,
                 editor_settings: ResolvedEditorSettings {
                     configuration: None,
                     lint_preview: Some(false),
@@ -619,7 +726,7 @@ mod tests {
                     exclude: None,
                     line_length: None,
                     configuration_preference: ConfigurationPreference::EditorFirst,
-                }
+                },
             }
         );
     }
@@ -673,6 +780,13 @@ mod tests {
                     ),
                 ),
                 configuration_preference: None,
+                show_syntax_errors: None,
+                tracing: TracingSettings {
+                    log_level: Some(
+                        Warn,
+                    ),
+                    log_file: None,
+                },
             },
         }
         "###);
@@ -693,6 +807,7 @@ mod tests {
                 lint_enable: true,
                 disable_rule_comment_enable: false,
                 fix_violation_enable: true,
+                show_syntax_errors: true,
                 editor_settings: ResolvedEditorSettings {
                     configuration: None,
                     lint_preview: None,
@@ -703,7 +818,7 @@ mod tests {
                     exclude: Some(vec!["third_party".into()]),
                     line_length: Some(LineLength::try_from(80).unwrap()),
                     configuration_preference: ConfigurationPreference::EditorFirst,
-                }
+                },
             }
         );
     }
@@ -714,4 +829,30 @@ mod tests {
 
         assert_eq!(options, InitializationOptions::default());
     }
+
+    fn assert_preview_client_settings(settings: &ClientSettings, preview: bool) {
+        assert_eq!(settings.lint.as_ref().unwrap().preview.unwrap(), preview);
+        assert_eq!(settings.format.as_ref().unwrap().preview.unwrap(), preview);
+    }
+
+    fn assert_preview_all_settings(all_settings: &AllSettings, preview: bool) {
+        assert_preview_client_settings(&all_settings.global_settings, preview);
+        if let Some(workspace_settings) = all_settings.workspace_settings.as_ref() {
+            for settings in workspace_settings.values() {
+                assert_preview_client_settings(settings, preview);
+            }
+        }
+    }
+
+    #[test]
+    fn test_preview_flag() {
+        let options = deserialize_fixture(EMPTY_MULTIPLE_WORKSPACE_INIT_OPTIONS_FIXTURE);
+        let mut all_settings = AllSettings::from_init_options(options);
+
+        all_settings.set_preview(false);
+        assert_preview_all_settings(&all_settings, false);
+
+        all_settings.set_preview(true);
+        assert_preview_all_settings(&all_settings, true);
+    }
 }
