@@ -1,6 +1,7 @@
 """
 Execution, comparison, and summary of `ruff check` ecosystem checks.
 """
+
 from __future__ import annotations
 
 import asyncio
@@ -35,7 +36,6 @@ if TYPE_CHECKING:
         Project,
     )
 
-
 # Matches lines that are summaries rather than diagnostics
 CHECK_SUMMARY_LINE_RE = re.compile(r"^(Found \d+ error.*)|(.* fixable with .*)$")
 
@@ -144,12 +144,32 @@ def markdown_check_result(result: Result) -> str:
 
         # Limit the number of items displayed per project to between 10 and 50
         # based on the proportion of total changes present in this project
-        max_display_per_project = max(10, int((project_changes / total_changes) * 50))
+        max_display_per_project = max(
+            10,
+            int(
+                (
+                    # TODO(zanieb): We take the `max` here to avoid division by zero errors where
+                    # `total_changes` is zero but `total_affected_rules` is non-zero so we did not
+                    # skip display. This shouldn't really happen and indicates a problem in the
+                    # calculation of these values. Instead of skipping entirely when `total_changes`
+                    # is zero, we'll attempt to report the results to help diagnose the problem.
+                    #
+                    # There's similar issues with the `max_display_per_rule` calculation immediately
+                    # below as well.
+                    project_changes / max(total_changes, 1)
+                )
+                * 50
+            ),
+        )
 
         # Limit the number of items displayed per rule to between 5 and the max for
         # the project based on the number of rules affected (less rules, more per rule)
         max_display_per_rule = max(
-            5, max_display_per_project // len(rule_changes.rule_codes())
+            5,
+            # TODO: remove the need for the max() call here,
+            # which is a workaround for if `len(rule_changes.rule_codes()) == 0`
+            # (see comment in the assignment of `max_display_per_project` immediately above)
+            max_display_per_project // max(len(rule_changes.rule_codes()), 1),
         )
 
         # Display the diff
